@@ -139,7 +139,7 @@ gnome_keyring_free_password (gchar *password)
 
 /**
  * gnome_keyring_string_list_free:
- * @strings: A %GList of string pointers.
+ * @strings: (element-type utf8): A %GList of string pointers.
  *
  * Free a list of string pointers.
  **/
@@ -223,9 +223,39 @@ gnome_keyring_found_free (GnomeKeyringFound *found)
 	g_free (found);
 }
 
+/**
+ * gnome_keyring_found_copy:
+ * @found: a #GnomeKeyringFound
+ *
+ * Copy a #GnomeKeyringFound item.
+ *
+ * Return value: (transfer full): The new #GnomeKeyringFound
+ */
+GnomeKeyringFound*
+gnome_keyring_found_copy (GnomeKeyringFound *found)
+{
+	GnomeKeyringFound *copy;
+
+	if (found == NULL)
+		return NULL;
+
+	copy = g_new (GnomeKeyringFound, 1);
+	copy->keyring = g_strdup (found->keyring);
+	copy->item_id = found->item_id;
+	copy->attributes = gnome_keyring_attribute_list_copy (found->attributes);
+	copy->secret = egg_secure_strdup (found->secret);
+
+	return copy;
+}
+
+G_DEFINE_BOXED_TYPE (GnomeKeyringFound,
+	gnome_keyring_found,
+	gnome_keyring_found_copy,
+	gnome_keyring_found_free);
+
 /**
  * gnome_keyring_found_list_free:
- * @found_list: a #GList of #GnomeKeyringFound
+ * @found_list: (element-type GnomeKeyringFound): a #GList of #GnomeKeyringFound
  *
  * Free the memory used by the #GnomeKeyringFound items in @found_list.
  **/
@@ -248,6 +278,96 @@ gnome_keyring_found_list_free (GList *found_list)
  * Each attribute has either a string, or unsigned integer value.
  **/
 
+/**
+ * gnome_keyring_attribute_get_string:
+ * @attribute: a #GnomeKeyringAttribute
+ *
+ * Return the string value. It is an error to call this method if
+ * @attribute.type is not #GNOME_KEYRING_ATTRIBUTE_TYPE_STRING. This method is
+ * mostly useful for language bindings which do not provide union access. In C
+ * you should just use attribute->value.string.
+ *
+ * Returns: (transfer none): The value.string pointer of @attribute. This is
+ * not a copy, do not free.
+ **/
+const gchar*
+gnome_keyring_attribute_get_string (GnomeKeyringAttribute *attribute)
+{
+	g_return_val_if_fail (attribute->type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING, NULL);
+	return attribute->value.string;
+}
+
+/**
+ * gnome_keyring_attribute_get_uint32:
+ * @attribute: a #GnomeKeyringAttribute
+ *
+ * Return the uint32 value. It is an error to call this method if
+ * @attribute.type is not #GNOME_KEYRING_ATTRIBUTE_TYPE_UINT32. This method is
+ * mostly useful for language bindings which do not provide union access. In C
+ * you should just use attribute->value.integer.
+ *
+ * Returns: The value.integer of @attribute.
+ **/
+guint32
+gnome_keyring_attribute_get_uint32 (GnomeKeyringAttribute *attribute)
+{
+	g_return_val_if_fail (attribute->type == GNOME_KEYRING_ATTRIBUTE_TYPE_UINT32, 0);
+	return attribute->value.integer;
+}
+
+/**
+ * gnome_keyring_attribute_free:
+ * @attribute: a #GnomeKeyringAttribute.
+ *
+ * Free the memory used by the #GnomeKeyringAttribute @attribute.
+ **/
+static void
+gnome_keyring_attribute_free (GnomeKeyringAttribute *attribute)
+{
+	if (attribute == NULL)
+		return;
+
+	g_free (attribute->name);
+	if (attribute->type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING) {
+		g_free (attribute->value.string);
+	}
+	g_free (attribute);
+}
+
+/**
+ * gnome_keyring_attribute_copy:
+ * @attribute: a #GnomeKeyringAttribute to copy.
+ *
+ * Copy a #GnomeKeyringAttribute.
+ *
+ * Return value: (transfer full): The new #GnomeKeyringAttribute
+ **/
+static GnomeKeyringAttribute*
+gnome_keyring_attribute_copy (GnomeKeyringAttribute *attribute)
+{
+	GnomeKeyringAttribute *copy;
+
+	if (attribute == NULL)
+		return NULL;
+
+	copy = g_new (GnomeKeyringAttribute, 1);
+	copy->name = g_strdup (attribute->name);
+	copy->type = attribute->type;
+	if (attribute->type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING) {
+		copy->value.string = g_strdup (attribute->value.string);
+	} else {
+		copy->value.integer = attribute->value.integer;
+	}
+
+	return copy;
+}
+
+G_DEFINE_BOXED_TYPE (GnomeKeyringAttribute,
+	gnome_keyring_attribute,
+	gnome_keyring_attribute_copy,
+	gnome_keyring_attribute_free);
+
+
 /**
  * gnome_keyring_attribute_list_append_string:
  * @attributes: A #GnomeKeyringAttributeList
@@ -295,6 +415,19 @@ gnome_keyring_attribute_list_append_uint32 (GnomeKeyringAttributeList *attribute
 	g_array_append_val (attributes, attribute);
 }
 
+/**
+ * gnome_keyring_attribute_list_new:
+ *
+ * Create a new #GnomeKeyringAttributeList.
+ *
+ * Return value: (transfer full): The new #GnomeKeyringAttributeList
+ **/
+GnomeKeyringAttributeList *
+gnome_keyring_attribute_list_new (void)
+{
+	return g_array_new (FALSE, FALSE, sizeof (GnomeKeyringAttribute));
+}
+
 /**
  * gnome_keyring_attribute_list_free:
  * @attributes: A #GnomeKeyringAttributeList
@@ -329,7 +462,7 @@ gnome_keyring_attribute_list_free (GnomeKeyringAttributeList *attributes)
  *
  * Copy a list of item attributes.
  *
- * Return value: The new #GnomeKeyringAttributeList
+ * Return value: (transfer full): The new #GnomeKeyringAttributeList
  **/
 GnomeKeyringAttributeList *
 gnome_keyring_attribute_list_copy (GnomeKeyringAttributeList *attributes)
@@ -356,6 +489,41 @@ gnome_keyring_attribute_list_copy (GnomeKeyringAttributeList *attributes)
 	return copy;
 }
 
+/**
+ * gnome_keyring_attribute_list_to_glist:
+ * @attributes: A #GnomeKeyringAttributeList
+ *
+ * Create #GList of #GnomeKeyringAttribute pointers from @attributes. This is
+ * mostly useful in language bindings which cannot directly use a #GArray.
+ *
+ * Returns: (transfer full) (element-type GnomeKeyringAttribute): #GList
+ * of #GnomeKeyringAttribute.
+ *
+ * Since: 3.4
+ **/
+GList*
+gnome_keyring_attribute_list_to_glist (GnomeKeyringAttributeList *attributes)
+{
+	GList *list = NULL;
+	GnomeKeyringAttribute *attr;
+	guint i;
+
+	if (attributes == NULL)
+		return NULL;
+
+	for (i = 0; i < attributes->len; ++i) {
+		attr = &g_array_index (attributes, GnomeKeyringAttribute, i);
+		list = g_list_append (list, gnome_keyring_attribute_copy (attr));
+	}
+
+	return list;
+}
+
+G_DEFINE_BOXED_TYPE (GnomeKeyringAttributeList,
+	gnome_keyring_attribute_list,
+	gnome_keyring_attribute_list_copy,
+	gnome_keyring_attribute_list_free);
+
 /**
  * SECTION:gnome-keyring-keyring-info
  * @title: Keyring Info
@@ -410,6 +578,11 @@ gnome_keyring_info_copy (GnomeKeyringInfo *keyring_info)
 	return copy;
 }
 
+G_DEFINE_BOXED_TYPE (GnomeKeyringInfo,
+	gnome_keyring_info,
+	gnome_keyring_info_copy,
+	gnome_keyring_info_free);
+
 /**
  * gnome_keyring_item_info_free:
  * @item_info: The keyring item info pointer.
@@ -473,6 +646,24 @@ gnome_keyring_item_info_copy (GnomeKeyringItemInfo *item_info)
 	return copy;
 }
 
+/* gnome_keyring_item_info_get_type() is already part of the API, cannot use
+ * G_DEFINE_BOXED_TYPE here */
+GType
+gnome_keyring_item_info_get_gtype (void)
+{
+    static volatile gsize initialized = 0;
+    static GType type = 0;
+
+    if (g_once_init_enter (&initialized)) {
+	type = g_boxed_type_register_static ("GnomeKeyringItemInfo",
+	    (GBoxedCopyFunc) gnome_keyring_item_info_copy,
+	    (GBoxedFreeFunc) gnome_keyring_item_info_free);
+	g_once_init_leave (&initialized, 1);
+    }
+
+    return type;
+}
+
 /**
  * gnome_keyring_application_ref_new:
  *
@@ -529,6 +720,11 @@ gnome_keyring_application_ref_copy (const GnomeKeyringApplicationRef *app)
 	return copy;
 }
 
+G_DEFINE_BOXED_TYPE (GnomeKeyringApplicationRef,
+	gnome_keyring_application_ref,
+	gnome_keyring_application_ref_copy,
+	gnome_keyring_application_ref_free);
+
 /**
  * gnome_keyring_access_control_new:
  * @application: A #GnomeKeyringApplicationRef pointer
@@ -590,14 +786,21 @@ gnome_keyring_access_control_copy (GnomeKeyringAccessControl *ac)
 	return ret;
 }
 
+G_DEFINE_BOXED_TYPE (GnomeKeyringAccessControl,
+	gnome_keyring_access_control,
+	gnome_keyring_access_control_copy,
+	gnome_keyring_access_control_free);
+
 /**
  * gnome_keyring_acl_copy:
- * @list: A list of #GnomeKeyringAccessControl pointers.
+ * @list: (element-type GnomeKeyringAccessControl): A list of
+ *        #GnomeKeyringAccessControl pointers.
  *
  * Copy an access control list.
  *
- * Return value: A new list of #GnomeKeyringAccessControl items. Use
- * gnome_keyring_acl_free() to free the memory.
+ * Return value: (transfer full) (element-type GnomeKeyringAccessControl):
+ * A new list of #GnomeKeyringAccessControl items. Use gnome_keyring_acl_free()
+ * to free the memory.
  **/
 GList *
 gnome_keyring_acl_copy (GList *list)
@@ -614,7 +817,8 @@ gnome_keyring_acl_copy (GList *list)
 
 /**
  * gnome_keyring_acl_free:
- * @acl: A list of #GnomeKeyringAccessControl pointers.
+ * @acl: (element-type GnomeKeyringAccessControl): A list of
+ *       #GnomeKeyringAccessControl pointers.
  *
  * Free an access control list.
  **/
